@@ -32,6 +32,7 @@ static struct option long_options[] =
 	{"junctionIns", required_argument, 0, 0},
 	{"multi",  required_argument, 0, 'B'},
 	{"exonAnnotation", required_argument, 0, 'a'},
+	{"exonAnnotationScreenOut", required_argument, 0, 0},
 	{"exonAlias", required_argument, 0, 'A'},
 	{"exonFormat", required_argument, 0, 'F'},
 	{"gtfAttr", required_argument, 0, 0},
@@ -113,13 +114,13 @@ void print_usage_core_subjunc()
 	SUBREADputs("");
 	SUBREADputs("# thresholds for mapping");
 	SUBREADputs("");
-	SUBREADputs("  -n <int>          Number of selected subreads, 10 by default.");
+	SUBREADputs("  -n <int>          Number of selected subreads, 14 by default.");
 	SUBREADputs("");
 	SUBREADputs("  -m <int>          Consensus threshold for reporting a hit (minimal number of");
 	SUBREADputs("                    subreads that map in consensus) . If paired-end, this gives");
 	SUBREADputs("                    the consensus threshold for the anchor read (anchor read");
 	SUBREADputs("                    receives more votes than the other read in the same pair).");
-	SUBREADputs("                    3 by default");
+	SUBREADputs("                    1 by default.");
 	SUBREADputs("");
 	SUBREADputs("  -p <int>          Consensus threshold for the non- anchor read in a pair. 1 by");
 	SUBREADputs("                    default.");
@@ -629,6 +630,9 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				{
 					global_context->config.max_vote_number_cutoff  = atoi(optarg);
 				}
+				else if(strcmp("exonAnnotationScreenOut", long_options[option_index].name)==0){
+					strcpy(global_context->config.exon_annotation_file_screen_out, optarg);
+				}
 				else if(strcmp("allJunctions", long_options[option_index].name)==0)
 				{
 					global_context->config.do_fusion_detection = 1;
