@@ -113,7 +113,7 @@ void add_chro(char *sam_h)
 	SUBREADprintf("Added a new chromosome : %s [%u]\n", chro_name, chro_len);
 }
 
-void get_read_info(char * fl, char * chro, unsigned int * pos_0base , char * cigar, int *flags, int * tlen, char ** rtext){
+void get_read_info(char * fl, char * chro, unsigned int * pos_1base , char * cigar, int *flags, int * tlen, char ** rtext){
 	char * tmp_tok = NULL;
 
 	char * tmp_res = strtok_r(fl, "\t", &tmp_tok);
@@ -125,7 +125,7 @@ void get_read_info(char * fl, char * chro, unsigned int * pos_0base , char * cig
 	strcpy(chro, tmp_res);
 
 	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // pos_0base
-	(*pos_0base) = atoi(tmp_res);
+	(*pos_1base) = atoi(tmp_res);
 
 	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // qual
 	tmp_res = strtok_r(NULL, "\t", &tmp_tok); // cigar
@@ -161,7 +161,11 @@ int covCalc()
 	coverage_bin_entry_t * chrbin12[2];
 	SamBam_FILE * in_fp = SamBam_fopen(input_file_name, is_BAM_input?SAMBAM_FILE_BAM:SAMBAM_FILE_SAM);
 	char * line_buffer = malloc(3000);
+	int this_hits = 0, r1_items = 0;
 	
+	char last_chro[200],chro[200], *rtext = NULL;
+	last_chro[0]=0;
+
 	while(1)
 	{
 		char * is_ret = SamBam_fgets(in_fp, line_buffer, 2999, 1);
@@ -177,31 +181,44 @@ int covCalc()
 	
 			int flags=0, x1, is_junc = 0, tlen = 0;
 			char cigar_str[200];
-			char chro[200], *rtext = NULL;
 			unsigned int pos_0base = 0;
 			cigar_str[0]=0;
+
+			if(!paired_end) this_hits = 0;
+			if(paired_end && (reads%2) == 0){
+				this_hits = 0;
+				r1_items = 0;
+				chrbin12[0] = chrbin12[1] = NULL;
+			}
+			if(paired_end && (reads%2) == 1)
+				strcpy(last_chro, chro);
+
 			chro[0]=0;
-			int this_hits = 0;
 
 			get_read_info(line_buffer, chro, &pos_0base, cigar_str, &flags, &tlen, &rtext);
 			pos_0base --;
 
 			if(0 && FIXLENstrcmp("NS500643:166:HNCCHBGXY:1:23105:1714:15599" , line_buffer)==0)
 				SUBREADprintf("INPUT READ_%d: FLAG=%d, chro=%s, pos=%u\n", 1+(reads % 2 ), flags, chro, pos_0base);
-			if((flags & 4) || ( paired_end && (tlen == 0 || abs(tlen) > 500000) )){
-				reads++;
-				continue;
-			}
+			//if((flags & 4) || ( paired_end && (tlen == 0 || abs(tlen) > 500000) )){
+			//	reads++;
+			//	continue;
+			//}
 
-			void ** bin_entry = HashTableGet(cov_bin_table, chro);
-			if(NULL == bin_entry)
-			{
-				SUBREADprintf("ERROR: The chromosome name is not in header:%s\n", chro);
-			}
+			coverage_bin_entry_t * chrbin = NULL;
+			unsigned int chrlen = 0;
+			int cigar_sections = 0;
+			if((flags & 4)==0){
+				void ** bin_entry = HashTableGet(cov_bin_table, chro);
+				if(NULL == bin_entry)
+				{
+					SUBREADprintf("ERROR: The chromosome name is not in header:%s\n", chro);
+				}
 
-			coverage_bin_entry_t * chrbin = (coverage_bin_entry_t*) bin_entry[0];
-			unsigned int chrlen = (void *)( bin_entry[1]) - NULL;
-			int cigar_sections = RSubread_parse_CIGAR_string(chro, pos_0base, cigar_str, max_M, Chros, Staring_Points, Staring_Read_Points, Section_Lengths, &is_junc);
+				chrbin = (coverage_bin_entry_t*) bin_entry[0];
+				chrlen = (void *)( bin_entry[1]) - NULL;
+				cigar_sections = RSubread_parse_CIGAR_string(chro, pos_0base, cigar_str, max_M, Chros, Staring_Points, Staring_Read_Points, Section_Lengths, &is_junc);
+			}
 			if(paired_end) {
 				for(x1 = 0; x1 < cigar_sections; x1++){
 					unsigned int x2,x3;
@@ -213,15 +230,18 @@ int covCalc()
 					int read_cursor = Staring_Read_Points[x1];
 					for(x2 = Staring_Points[x1]; x2<Staring_Points[x1]+Section_Lengths[x1]; x2++){
 						int found = 0;
-						for(x3=0; x3<this_hits; x3++){
-							if(this_hit_locs[x3] == x2){
-								found =1;
-								break;
+
+						if(reads%2 == 1 && strcmp(chro,last_chro)==0)
+							for(x3=0; x3<this_hits; x3++){
+								if(this_hit_locs[x3] == x2){
+									found =1;
+									break;
+								}
 							}
-						}
 						if(!found){
 							this_hit_bases[this_hits] = base_value_to_int(rtext[read_cursor]);
 							this_hit_locs[this_hits++] = x2;
+							if(reads % 2 == 0) r1_items ++;
 						}
 						if(this_hits >= MAX_FRAGMENT_LENGTH){
 							SUBREADprintf("ERROR: read is too long : %s!\n", cigar_str);
@@ -231,25 +251,14 @@ int covCalc()
 					}
 				}
 
-				if(this_hits > 0) assert(chrbin);
 				chrbin12[reads%2] = chrbin;
 			} else for(x1 = 0; x1 < cigar_sections; x1++) {
 				unsigned int x2;
-				if(0 && FIXLENstrcmp("NS500643:166:HNCCHBGXY:4:23507:11772:18798" , line_buffer)==0)
-					SUBREADprintf("TEST_ONE %s [%d] : %u ~ %u  (chr_len=%u)  ; read_seq = [%d]  %s\n", line_buffer, x1, Staring_Points[x1], x2<Staring_Points[x1]+Section_Lengths[x1], chrlen,  Staring_Read_Points[x1], rtext +  Staring_Read_Points[x1]);
-
-				if(0 && strcmp("chr12",chro) == 0 && Staring_Points[x1] <= 113994516 && Staring_Points[x1] + Section_Lengths[x1] > 113994516){
-					int rpos_0 = 113994516 - Staring_Points[x1] + Staring_Read_Points[x1];
-					SUBREADprintf("    TEST_BASE : %s : val=%c\n", line_buffer, rtext[rpos_0]);
-				}
-
 				int read_cursor = Staring_Read_Points[x1];
 				for(x2 = Staring_Points[x1]; x2<Staring_Points[x1]+Section_Lengths[x1]; x2++) {
 					if(x2 < chrlen) {
 						if(base_values){
 							chrbin[x2*5 + base_value_to_int(rtext[read_cursor])] ++;
-							if(0 && FIXLENstrcmp("NS500643:166:HNCCHBGXY:4:23507:11772:18798" , line_buffer)==0)
-								SUBREADprintf("    TEST_MY_BASE : [%d 0_base] = %c (%d) ; res=%d\n", x2, rtext[read_cursor],  base_value_to_int(rtext[read_cursor]), chrbin[x2*5 + base_value_to_int(rtext[read_cursor])]);
 						}
 						else if(chrbin[x2] <= COVERAGE_MAX_INT)chrbin[x2] ++;
 						all_counted ++;
@@ -263,14 +272,17 @@ int covCalc()
 					read_cursor ++;
 				}
 			}
-			if(reads % 2 == 1){
-				int r,x1;
-				for(r = 0; r < 2; r++){
-					if(this_hits>0)assert(chrbin12[r]);
-					for(x1 = 0; x1<this_hits;x1++){
-						if(base_values) chrbin12[r][this_hit_locs[x1]*5 + this_hit_bases[x1]]++;
-						else chrbin12[r][this_hit_locs[x1]]++;
-					}
+			//SUBREADprintf("SREAD_%d , R1ITEMS=%d, R1+2ITEMS=%d, BIN1=%p, BIN2=%p\n", reads, r1_items, this_hits, chrbin12[0], chrbin12[1]);
+			if((reads % 2) == 1 && paired_end){
+				int x1;
+				for(x1 = 0; x1<this_hits;x1++){
+					coverage_bin_entry_t * this_ch_binn = x1 < r1_items?chrbin12[0]:chrbin12[1];
+					if(base_values) this_ch_binn[this_hit_locs[x1]*5 + this_hit_bases[x1]]++;
+					else this_ch_binn[this_hit_locs[x1]]++;
+
+					all_counted ++;
+					if(all_counted % 10000000 == 0)
+						SUBREADprintf("Processed %llu bases.\n", all_counted);
 				}
 			}
 			reads ++;
