@@ -64,6 +64,7 @@ static struct option GRA_long_options[] =
 	{"hugeMemory", no_argument, 0, 'H'},
 	{"tmpDir",required_argument, 0, 't'},
 	{"trimQuality", required_argument, 0, 'Q'},
+	{"ignorePairedNames", no_argument, 0, 'C'},
 	{0, 0, 0, 0}
 };
 
@@ -111,6 +112,7 @@ typedef struct {
 	int maximum_mismatch_in20bp;
 	int reverse_unmapped_reads;
 	int phred_offset;
+	int check_read_names;
 	unsigned int maximum_contig_length;
 	unsigned int minimum_contig_length;
 
@@ -259,6 +261,7 @@ int GRA_check_config (GRA_global_context_t * global_context)
 
 void GRA_init_context(GRA_global_context_t * global_context)
 {
+	char mac_rand[13];
 	FILE * system_random_fp; 
 	memset(global_context, 0 , sizeof(GRA_global_context_t));
 
@@ -267,7 +270,9 @@ void GRA_init_context(GRA_global_context_t * global_context)
 	assert(system_random_fp);
 	fread(global_context -> random_seeds, 8 * sizeof(int), 1, system_random_fp);
 	fclose(system_random_fp);
-	sprintf(global_context -> tmp_file_prefix, "GRAtmp-%u-%d", global_context -> random_seeds[0], global_context -> system_pid);
+
+	mac_or_rand_str(mac_rand);
+	sprintf(global_context -> tmp_file_prefix, "GRAtmp-%06d-%s", global_context -> system_pid, mac_rand);
 
 	// best for simulation = 40
 	global_context -> shortest_read_allowed = 40; 
@@ -276,18 +281,22 @@ void GRA_init_context(GRA_global_context_t * global_context)
 
 	global_context -> total_threads = 1;
 
-	global_context -> maximum_mismatch_in20bp = 0;
+	//#warning "================ REMOVE ' + 1' FROM THE NEXT LINE ===================="
+	global_context -> maximum_mismatch_in20bp = 0 + 1;
 	global_context -> phred_offset = 33;
-	global_context -> read_trim_base_quality = 30 + 33; 
+	global_context -> read_trim_base_quality = 30 + global_context -> phred_offset; 
 
-	global_context -> subread_length = 16;
+	//#warning "================ REMOVE ' - 3' FROM THE NEXT LINE ===================="
+	global_context -> subread_length = 16 - 3;
 	global_context -> subread_extract_step = 5;
 
 
-	global_context -> min_overlap_votes = 4;
+	//#warning "================ REMOVE ' - 2' FROM THE NEXT LINE ===================="
+	global_context -> min_overlap_votes = 4 - 2;
 
 
-	global_context -> min_extension_votes = 3;
+	//#warning "================ REMOVE ' - 1' FROM THE NEXT LINE ===================="
+	global_context -> min_extension_votes = 3 - 1;
 	global_context -> contig_str = NULL;
 	global_context -> start_time = miltime();
 	global_context -> maximum_contig_length = 50000000;
@@ -295,13 +304,13 @@ void GRA_init_context(GRA_global_context_t * global_context)
 
 	global_context -> minimum_contig_length = 251;
 	global_context -> reverse_unmapped_reads = 0;
+	global_context -> check_read_names = 1;
 
 
 
 	global_context -> debug_flags = 0*GRA_DEBUG_ASSEMBLING;
 }
 
-
 int GRA_scan_best_overlap(GRA_global_context_t * global_context, char * piece, char * read, int rlen)
 {
 	int xk1;
@@ -318,6 +327,8 @@ int GRA_scan_best_overlap(GRA_global_context_t * global_context, char * piece, c
 	return best_loc;
 }
 
+
+
 // r1 is a zero-terminating string
 // r2 is not a zero-terminating string.
 // r1 and r2 are on opposite strands.
@@ -458,7 +469,10 @@ int GRA_trim_reads(GRA_global_context_t * global_context)
 		
 
 		if(in_flags & SAM_FLAG_SECOND_READ_IN_PAIR)
-			assert(readname_hash == fc_chro_hash(in_readname));
+		{
+			if(global_context -> check_read_names)
+				assert(readname_hash == fc_chro_hash(in_readname));
+		}
 		else{
 			unsigned long new_readname_hash = fc_chro_hash(in_readname); 
 			readname_hash = new_readname_hash;
@@ -1743,7 +1757,7 @@ int main(int argc, char ** argv)
 	}
 
 	char * tmptmp = malloc(320);
-	while ((c = getopt_long (argc, argv, "2:t:Q:V:L:i:I:o:bHR6", GRA_long_options, &option_index)) != -1)
+	while ((c = getopt_long (argc, argv, "2:t:Q:V:L:i:I:o:bHCR6", GRA_long_options, &option_index)) != -1)
 		switch(c)
 		{
 			case '2':
@@ -1791,6 +1805,9 @@ int main(int argc, char ** argv)
 			case 'Q':
 				tmp_trim_qual = atoi(optarg);
 			break;
+			case 'C':
+				global_context -> check_read_names = 0;
+			break;
 
 			default:
 				GRA_print_usage();
