@@ -239,7 +239,7 @@ FILE * get_FP_by_read_name(propMapped_context * context, char * read_name)
 	FILE * ret = HashTableGet(context -> split_fp_table, NULL+1+(int)(hash_key));
 	if(!ret)
 	{
-		char fname [300];
+		char fname [340];
 		sprintf(fname, "%s-%llu.bin", context->temp_file_prefix, hash_key);
 		ret = f_subr_open(fname, "wb");
 		HashTablePut(context -> split_fp_table, NULL+1+(int)(hash_key), ret);
@@ -287,7 +287,9 @@ int init_PE_sambam(propMapped_context * context)
 {
 	char mac_rand[13];
 	mac_or_rand_str(mac_rand);
+	#ifdef MAKE_STANDALONE
 	srand(time(NULL));
+	#endif
 
 	int x1;
 	context->temp_file_prefix[0]=0;
@@ -327,6 +329,7 @@ int split_PE_sambam(propMapped_context * context)
 		if(!is_ret) break;
 		if(line_buffer[0]=='@') continue;
 
+		//SUBREADprintf("TELLO: %ld\n", ftello(in_fp -> os_file));
 		char * tok_tmp;
 		char * read_name = strtok_r (line_buffer,"\t", &tok_tmp);	// read name
 		char * flags_str = strtok_r (NULL ,"\t", &tok_tmp);
@@ -351,7 +354,7 @@ int prop_PE(propMapped_context * context)
 	int bini;
 	for(bini = 0; bini < context -> sort_buckets; bini++)
 	{
-		char fname[300];
+		char fname[360];
 		HashTable * rname_table = HashTableCreate(100000);
 		HashTableSetKeyComparisonFunction(rname_table , fc_strcmp_chro);
 		HashTableSetHashFunction(rname_table, fc_chro_hash);
@@ -366,14 +369,15 @@ int prop_PE(propMapped_context * context)
 					unsigned char read_len;
 					unsigned short flags;
 					char * read_name;
-					fread(&read_len,1,1,fp);
+					int retvv = fread(&read_len,1,1,fp);
+					if(retvv<1) break;
 					if(feof(fp))break;
 					read_name = malloc((int)read_len+1);
-					fread(read_name,read_len,1,fp);
+					retvv = fread(read_name,read_len,1,fp);
+					if(retvv<1) break;
 					read_name[read_len]=0;
-					//assert(read_len == strlen(read_name));
-					//printf("RNAME=%s;\n", read_name);
-					fread(&flags,1, sizeof(short), fp);
+					retvv = fread(&flags,1, sizeof(short), fp);
+					if(retvv<1) break;
 
 					int new_OK; 
 					
@@ -421,6 +425,7 @@ int prop_PE(propMapped_context * context)
 					}
 				}
 		}
+		//SUBREADprintf("BIN %d  ALLR %llu  MAPPED %llu  ELES=%ld\n", bini, context -> all_reads, context -> mapped_reads, rname_table -> numOfElements);
 		HashTableDestroy(rname_table);
 	}
 	return 0;
