@@ -8,7 +8,7 @@
 #include "subread.h"
 
 // returns 0 if OK; returns 1 if the file is not indexable; returns -1 if file doesn't exist.
-int seekgz_open(const char * fname, seekable_zfile_t * fp);
+int seekgz_open(const char * fname, seekable_zfile_t * fp, FILE * old_fp);
 
 // returns length in bytes if OK (length includes the line break at the end); returns 0 if EOF
 int seekgz_gets(seekable_zfile_t * fp, char * buf, int buf_size);
@@ -27,6 +27,8 @@ typedef struct {
 	int is_plain;
 	FILE * plain_fp;
 	seekable_zfile_t gz_fp;
+	int is_first_chars;
+	char first_chars[2];
 } autozip_fp;
 
 // returns length in bytes if OK (length includes the line break at the end); returns 0 if EOF
@@ -38,4 +40,8 @@ void autozip_close(autozip_fp * fp);
 int autozip_open(const char * fname, autozip_fp * fp);
 
 void autozip_rewind(autozip_fp * fp);
+
+int seekgz_preload_buffer( seekable_zfile_t * fp , subread_lock_t * read_lock);
+
+int seekgz_gets(seekable_zfile_t * fp, char * buff, int buff_len);
 #endif
