@@ -79,7 +79,7 @@ int main(int argc, char ** argv)
 
 	if(in_SAM && out_SAM && !sort_needed)
 	{
-		SUBREADprintf("Subtools can convert between SAM and BAM files and/or re-order the alignment results by the read names.\n");
+		SUBREADprintf("Subtools can convert between SAM and BAM files and/or pair up the alignment results by the read names.\n");
 		SUBREADprintf("No operation was specified. The output file was not generated.\n");
 		return 0;
 	}
@@ -165,7 +165,7 @@ int main(int argc, char ** argv)
 		SUBREADprintf("Loading time: %.2f, Sorting time: %.2f\n", t1-t0, t2-t1);
 
 		if(writer.unpaired_reads && 0)
-		SUBREADprintf("%llu reads were re-ordered.\n", writer.written_reads);
+		SUBREADprintf("%llu reads were paired up.\n", writer.written_reads);
 
 		SamBam_fclose(in_fp);
 
